"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadAssets = exports.generateWebpackWebDevServerFile = exports.getMaterialNodeModulesPathList = exports.installDependencies = exports.downloadDependencies = exports.downloadAndInstallDependencies = exports.getAllPageMpEntryPath = exports.getWebpackMpBuildParams = exports.getWebpackWebBuildParams = exports.getMpAllRouterConfig = exports.getPageName = exports.getHomePageInstance = exports.getMainAppDataByList = exports.generateMpJsonConfigFile = exports.generateKboneAppConfig = exports.generateKbonePageConfig = exports.downloadAndWriteTabBarIcon = exports.generateKboneTabBarConfig = exports.extractAndRemoveKbConfig = exports.generateWebpackWebBuildParamsFile = exports.fixAppJson = exports.startCompile = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const webpack_1 = __importDefault(require("webpack"));
const lodash_template_1 = __importDefault(require("lodash.template"));
const util_1 = require("../../util");
const process_1 = require("../../util/process");
const axios_1 = __importDefault(require("axios"));
const compressing_1 = __importDefault(require("compressing"));
const cross_spawn_1 = __importDefault(require("cross-spawn"));
const weapps_core_1 = require("../../../weapps-core");
const config_1 = require("../../config");
const plugin_1 = require("./plugin");
const command_exists_1 = require("command-exists");
const common_1 = require("../../util/common");
const common_2 = require("../../types/common");
const config_2 = require("../../config");
const console_1 = require("../../util/console");
const types_1 = require("../../../types");
const yarnExists = command_exists_1.sync('yarn');
const runningCompilations = {};
function startCompile(options, cb) {
    const key = options.configPath;
    const runningProcess = runningCompilations[key];
    if (runningProcess) {
        console.log(`Compiling ${options.configPath} already running`);
        return;
    }
    console.log('Running webpack by ' + options.configPath);
    util_1.removeRequireUncached(path_1.default.resolve(options.configPath, '../miniprogram.config.js'));
    const watching = webpack_1.default(util_1.requireUncached(options.configPath), (err, stats) => __awaiter(this, void 0, void 0, function* () {
        if (err) {
            console.error('webpack config error', err.stack || err);
            if (err.details) {
                console.error('webpack config error detail', err.details);
            }
            cb === null || cb === void 0 ? void 0 : cb(err);
            delete runningCompilations[key];
            return;
        }
        const info = stats.toJson('minimal');
        if (stats.hasErrors()) {
            console.error('Webpack compilation errors', info.errors.join('\n'));
            cb === null || cb === void 0 ? void 0 : cb(info.errors);
        }
        else {
            let { endTime = 0, startTime = 0 } = stats;
            cb === null || cb === void 0 ? void 0 : cb(null, {
                outDir: options.appBuildDir,
                timeElapsed: endTime - startTime,
            });
        }
        if (stats.hasWarnings()) {
            console.warn('webpack compiling warnings', info.warnings.join('\n'));
        }
    }));
    if (!(watching instanceof webpack_1.default.Compiler)) {
        runningCompilations[key] = watching;
    }
}
exports.startCompile = startCompile;
function fixAppJson(appBuildDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const appJsonPath = path_1.default.resolve(appBuildDir, 'dist/mp/app.json');
        if (!fs_extra_1.default.existsSync(appJsonPath)) {
            return;
        }
        const appJson = yield fs_extra_1.default.readJSON(appJsonPath);
        appJson.subpackages = appJson.subpackages.map((config) => {
            config.pages = config.pages.map((pagePath) => {
                const pagePathArr = pagePath.split('/');
                if (pagePathArr.length === 4 && pagePathArr[1] === config.root) {
                    pagePathArr.splice(1, 1);
                    return pagePathArr.join('/');
                }
                return pagePath;
            });
            return config;
        });
        yield fs_extra_1.default.writeFile(appJsonPath, JSON.stringify(appJson, null, 2));
    });
}
exports.fixAppJson = fixAppJson;
function generateWebpackWebBuildParamsFile({ appId, allAppDataList, appBuildDir, materialsDir, dependencies = [], nodeModulesPath, publicPath, mode, watch, buildTypeList, assets = [], }) {
    return __awaiter(this, void 0, void 0, function* () {
        let mainAppData = getMainAppDataByList(allAppDataList);
        let extraDefine = {
            'process.env.historyType': `"${mainAppData.historyType || types_1.HISTORY_TYPE.BROWSER}"`,
        };
        const params = getWebpackWebBuildParams(appId, appBuildDir, materialsDir, dependencies, nodeModulesPath, publicPath, mode, watch, buildTypeList, extraDefine, assets);
        const webpackConfigPath = path_1.default.resolve(appBuildDir, './webpack/webpack.web.prod.js');
        const paramsString = JSON.stringify(params, null, 2);
        const webpackConfigContent = `const params = ${paramsString};\nmodule.exports = require('./web.prod.js')(params);`;
        yield fs_extra_1.default.writeFile(webpackConfigPath, webpackConfigContent);
        return webpackConfigPath;
    });
}
exports.generateWebpackWebBuildParamsFile = generateWebpackWebBuildParamsFile;
function extractAndRemoveKbConfig(mainAppData, subAppDataList, appBuildDir) {
    return __awaiter(this, void 0, void 0, function* () {
        let originMpConfig = {};
        const configMod = mainAppData.codeModules.find((p) => p.name === config_1.MP_CONFIG_MODULE_NAME);
        if (configMod) {
            const code = configMod.code.replace(/export\s+default/, '');
            try {
                originMpConfig = eval(`(${code})`);
            }
            catch (e) {
                console.error('Kbone config file error', e);
            }
        }
        generateKboneAppConfig(originMpConfig, mainAppData);
        originMpConfig = generateKbonePageConfig(originMpConfig, mainAppData, subAppDataList);
        yield generateKboneTabBarConfig(originMpConfig, appBuildDir);
        return originMpConfig;
    });
}
exports.extractAndRemoveKbConfig = extractAndRemoveKbConfig;
function generateKboneTabBarConfig(mpConfig, appBuildDir) {
    return __awaiter(this, void 0, void 0, function* () {
        mpConfig.appExtraConfig = mpConfig.appExtraConfig || {};
        if (mpConfig.appExtraConfig.tabBar) {
            if (mpConfig.appExtraConfig.tabBar.list) {
                yield Promise.all(mpConfig.appExtraConfig.tabBar.list.map((item) => __awaiter(this, void 0, void 0, function* () {
                    if (item.iconPath) {
                        item.iconPath = yield downloadAndWriteTabBarIcon(item.iconPath, item.pagePath, appBuildDir, 'iconPath');
                    }
                    if (item.selectedIconPath) {
                        item.selectedIconPath = yield downloadAndWriteTabBarIcon(item.selectedIconPath, item.pagePath, appBuildDir, 'selectedIconPath');
                    }
                })));
            }
        }
    });
}
exports.generateKboneTabBarConfig = generateKboneTabBarConfig;
function downloadAndWriteTabBarIcon(iconPath, pagePath, appBuildDir, fileName) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('开始下载文件', iconPath);
        const extname = path_1.default.extname(iconPath);
        const relativePath = `images_tabBar/${pagePath}/${fileName}${extname}`;
        const fileFullPath = path_1.default.resolve(appBuildDir, `dist/mp/${relativePath}`);
        yield fs_extra_1.default.ensureFile(fileFullPath);
        const writer = fs_extra_1.default.createWriteStream(fileFullPath);
        const response = yield axios_1.default({
            method: 'get',
            url: iconPath,
            responseType: 'stream',
        });
        response.data.pipe(writer);
        return new Promise((resolve, reject) => {
            writer.on('finish', () => {
                console.log('下载成功', iconPath);
                resolve(relativePath);
            });
            writer.on('error', () => {
                console.error('下载失败', iconPath);
                reject();
            });
        });
    });
}
exports.downloadAndWriteTabBarIcon = downloadAndWriteTabBarIcon;
function generateKbonePageConfig(mpConfig, mainAppData, subAppDataList = []) {
    mpConfig.pages = mpConfig.pages || {};
    const allAppDataList = subAppDataList.concat(mainAppData);
    allAppDataList.map((appData) => {
        appData.pageInstanceList.forEach((item) => {
            const pageId = [appData.rootPath, item.id].filter((i) => i).join('_');
            if (!mpConfig.pages[pageId]) {
                mpConfig.pages[pageId] = {};
            }
            const navigationBarTitleText = item.data.navigationBarTitleText || item.data.title;
            delete item.data.title;
            delete item.data.params;
            delete item.data.scene;
            config_1.KBONE_PAGE_KEYS.forEach((key) => {
                if (item.data[key]) {
                    mpConfig.pages[pageId][key] = item.data[key];
                    delete item.data[key];
                }
            });
            mpConfig.pages[pageId].extra = Object.assign(Object.assign({}, item.data), { navigationBarTitleText });
        });
    });
    return mpConfig;
}
exports.generateKbonePageConfig = generateKbonePageConfig;
function generateKboneAppConfig(mpConfig, mainAppData) {
    if (mainAppData === null || mainAppData === void 0 ? void 0 : mainAppData.appConfig) {
        if (mainAppData.appConfig.window) {
            mpConfig.app = mpConfig.app || {};
            mpConfig.app = Object.assign(Object.assign({}, mainAppData.appConfig.window), mpConfig.app);
        }
    }
}
exports.generateKboneAppConfig = generateKboneAppConfig;
function generateMpJsonConfigFile(allAppDataList, userConfig, appBuildDir, plugins, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const mainAppData = getMainAppDataByList(allAppDataList);
        const subAppDataList = allAppDataList.filter((i) => i.rootPath);
        const homeId = getHomePageInstance(mainAppData.pageInstanceList).id;
        userConfig = userConfig || {};
        const kbConfig = Object.assign(Object.assign({ origin: 'https://weapps.tencent.com', entry: `/${homeId}`, redirect: {
                notFound: `${homeId}`,
                accessDenied: `${homeId}`,
            }, generate: {
                autoBuildNpm: true,
                appEntry: 'miniprogram-app',
                appWxss: 'display',
                subpackages: {},
                globalVars: [['__injectProxy', '(window.Proxy = Proxy)']],
            }, app: {
                navigationBarTitleText: `WeApps-${options.appKey}`,
            }, projectConfig: {
                appid: 'touristappid',
                projectname: `WeApps-${options.appKey}`,
            }, packageConfig: {
                author: 'weapps',
            } }, userConfig), { global: Object.assign(Object.assign({}, userConfig.global), { rem: true }), router: getMpAllRouterConfig(allAppDataList), __homePath__: getMpAllRouterConfig(allAppDataList, true) });
        if (subAppDataList === null || subAppDataList === void 0 ? void 0 : subAppDataList.length) {
            const subpackages = {};
            subAppDataList.map((appData) => {
                const { rootPath } = appData;
                subpackages[rootPath] = Object.keys(getMpAllRouterConfig([appData]));
            });
            kbConfig.generate = Object.assign(Object.assign({}, kbConfig.generate), { subpackages });
        }
        if (options.generateMpType === 'subpackage') {
            kbConfig.generate = Object.assign(Object.assign({}, kbConfig.generate), { app: 'noemit' });
        }
        if (plugins.length > 0) {
            kbConfig.generate.subpackages = Object.assign(Object.assign({}, kbConfig.generate.subpackages), (yield plugin_1.getPluginKboneSubpackage(appBuildDir, plugins)));
        }
        const templateStr = JSON.stringify(kbConfig, null, 2);
        yield fs_extra_1.default.writeFile(path_1.default.resolve(appBuildDir, 'webpack/miniprogram.config.js'), `module.exports = ${templateStr}`);
    });
}
exports.generateMpJsonConfigFile = generateMpJsonConfigFile;
function getMainAppDataByList(allAppDataList) {
    return allAppDataList.find((item) => !item.rootPath);
}
exports.getMainAppDataByList = getMainAppDataByList;
function getHomePageInstance(pageInstanceList) {
    let target = pageInstanceList[0];
    weapps_core_1.loopDealWithFn(pageInstanceList, (pageInstance) => {
        if (pageInstance.isHome) {
            target = pageInstance;
            return pageInstance;
        }
    });
    return target;
}
exports.getHomePageInstance = getHomePageInstance;
function getPageName(name) {
    return `${name}`;
}
exports.getPageName = getPageName;
function getMpAllRouterConfig(allAppDataList, getHome = false) {
    const router = {};
    let homePath = '';
    allAppDataList.map((appData) => {
        const { pageInstanceList, rootPath = '' } = appData;
        weapps_core_1.loopDealWithFn(pageInstanceList, (pageInstance) => {
            const name = [rootPath, pageInstance.id].filter((i) => i).join('_');
            const path = `/${name}`;
            if (!homePath) {
                homePath = name;
            }
            if (pageInstance.isHome && !rootPath) {
                homePath = name;
            }
            router[name] = [path];
        });
    });
    if (getHome) {
        return homePath;
    }
    return router;
}
exports.getMpAllRouterConfig = getMpAllRouterConfig;
function getWebpackWebBuildParams(appId, appBuildDir, materialsDir, dependencies = [], nodeModulesPath, publicPath = '/', mode = "", watch = false, buildTypeList = ["web"], extraDefine = {}, assets = []) {
    let jsApis = [];
    fs_extra_1.default.ensureDir(path_1.default.resolve(appBuildDir, 'assets'));
    if (buildTypeList.includes("qywxH5")) {
        jsApis = ['//res.wx.qq.com/open/js/jweixin-1.2.0.js'];
    }
    else if (buildTypeList.includes("wxH5")) {
        jsApis = ['//res.wx.qq.com/open/js/jweixin-1.6.0.js'];
    }
    if (assets && assets.length > 0) {
        if (buildTypeList.includes("app")) {
            const targetDir = path_1.default.resolve(appBuildDir, './assets');
            assets
                .filter((assetUrl) => !!assetUrl)
                .forEach((assetUrl) => __awaiter(this, void 0, void 0, function* () {
                const fileName = common_1.getFileNameByUrl(assetUrl);
                jsApis.push(`./${fileName}`);
                yield downloadAssets(targetDir, assetUrl);
            }));
        }
        else {
            jsApis = jsApis.concat(assets);
        }
    }
    return {
        context: appBuildDir,
        mode: mode !== "production" ? 'development' : mode,
        watch,
        entry: path_1.default.resolve(appBuildDir, 'src/index.jsx'),
        output: {
            path: path_1.default.resolve(appBuildDir, './preview'),
            filename: mode !== 'production'
                ? '[name].bundle.js'
                : '[name].[contenthash].bundle.js',
            chunkFilename: mode !== 'production'
                ? '[name].chunk.js'
                : '[name].[contenthash].chunk.js',
            publicPath: buildTypeList.includes("app") ||
                buildTypeList.includes("adminPortal")
                ? ''
                : publicPath,
            pathinfo: false,
        },
        htmlTemplatePath: path_1.default.resolve(appBuildDir, './html/index.html.ejs'),
        htmlTemplateData: {
            appId,
            meta: {
                jsApis,
            },
            delevopment: mode !== "production",
            isAdminPortal: buildTypeList.includes("adminPortal"),
        },
        externals: {
            mobx: 'window.mobx',
            '@cloudbase/js-sdk': 'window.cloudbase',
            '@cloudbase/weda-cloud-sdk/dist/h5': 'window.CloudSDK',
            '@zxing/library': 'window.ZXing',
        },
        resolveModules: [
            path_1.default.resolve(appBuildDir, 'node_modules'),
            path_1.default.resolve(appBuildDir, 'src'),
            ...getMaterialNodeModulesPathList(dependencies, materialsDir),
        ],
        definePlugin: Object.assign({ 'process.env.buildType': `"${buildTypeList[0]}"`, 'process.env.isApp': buildTypeList.includes("app"), 'process.env.isAdminPortal': common_2.buildAsAdminPortalByBuildType(buildTypeList) }, extraDefine),
    };
}
exports.getWebpackWebBuildParams = getWebpackWebBuildParams;
function getWebpackMpBuildParams(appBuildDir, materialsDir, dependencies = [], nodeModulesPath, allAppDataList, mode, watch, options) {
    return {
        context: appBuildDir,
        mode,
        watch,
        entry: getAllPageMpEntryPath(allAppDataList, appBuildDir, options),
        outputPath: path_1.default.resolve(appBuildDir, 'dist/mp/common'),
        resolveModules: [
            appBuildDir,
            path_1.default.resolve(appBuildDir, 'node_modules'),
            path_1.default.resolve(appBuildDir, 'src'),
            ...getMaterialNodeModulesPathList(dependencies, materialsDir),
        ],
        watchOptions: {
            ignored: /node_modules/,
            aggregateTimeout: 600,
            poll: 1000,
        },
        cache: {
            type: 'memory',
        },
    };
}
exports.getWebpackMpBuildParams = getWebpackMpBuildParams;
function getAllPageMpEntryPath(allAppDataList, appBuildDir, options = { generateMpType: 'app' }) {
    const entry = {};
    if (options.generateMpType === 'app') {
        entry['miniprogram-app'] = path_1.default.resolve(appBuildDir, './src/miniprogram-app.js');
    }
    const mainAppData = getMainAppDataByList(allAppDataList);
    const homePageInstance = getHomePageInstance(mainAppData.pageInstanceList);
    entry[homePageInstance.id] = path_1.default.resolve(appBuildDir, `src/pages/${getPageName(homePageInstance.id)}/main.mp.jsx`);
    allAppDataList.map((app) => {
        const { pageInstanceList, rootPath } = app;
        const packagePathStr = rootPath ? `packages/${rootPath}` : '';
        weapps_core_1.loopDealWithFn(pageInstanceList, (pageInstance) => {
            const pageName = rootPath
                ? `${rootPath}_${pageInstance.id}`
                : pageInstance.id;
            entry[pageName] = path_1.default.resolve(appBuildDir, path_1.default.join('src', packagePathStr, `pages/${getPageName(pageInstance.id)}/main.mp.jsx`));
        });
    });
    return entry;
}
exports.getAllPageMpEntryPath = getAllPageMpEntryPath;
const dependenciesMap = new Map();
function downloadAndInstallDependencies(dependencies = [], materialsDir, installOptions = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const localPkg = util_1.getCurrentPackageJson();
        yield Promise.all(dependencies.map(({ name, version, srcZipUrl }) => __awaiter(this, void 0, void 0, function* () {
            const materialNameVersion = `${name}@${version}`;
            const targetDir = path_1.default.join(materialsDir, materialNameVersion);
            if (localPkg && localPkg.name === name && localPkg.version === version) {
                console.log('当前本地目录是素材库的时候，无需安装', materialNameVersion);
                return;
            }
            if (dependenciesMap.get(targetDir)) {
                console_1.notice(`${materialNameVersion} 存在 ${targetDir}，无需安装, 如果依赖库报错，请重启wa watch`);
                return;
            }
            yield downloadDependencies(targetDir, srcZipUrl);
            yield installDependencies(targetDir, Object.assign(Object.assign({}, installOptions), { isDependence: true, ignoreInstall: common_1.isOfficialComponentLib(name, version) }));
            dependenciesMap.set(targetDir, true);
        })));
    });
}
exports.downloadAndInstallDependencies = downloadAndInstallDependencies;
function downloadDependencies(targetDir, srcZipUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        const isExist = fs_extra_1.default.existsSync(path_1.default.join(targetDir, 'package.json'));
        if (isExist) {
            return;
        }
        try {
            let response = yield axios_1.default({
                url: srcZipUrl,
                responseType: 'stream',
            });
            yield fs_extra_1.default.ensureDir(targetDir);
            yield compressing_1.default.zip.uncompress(response.data, targetDir);
        }
        catch (e) {
            console.error('Fail to download weapps material package ' + srcZipUrl, e);
            throw e;
        }
    });
}
exports.downloadDependencies = downloadDependencies;
function installDependencies(targetDir, options = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        if (options === null || options === void 0 ? void 0 : options.ignoreInstall) {
            console.log('ignore install dependencies in ' + targetDir);
            return;
        }
        let packageName = options.packageName || '';
        if (packageName && options.latest) {
            packageName = `${packageName}@latest`;
        }
        console.log('Installing dependencies in ' + targetDir);
        const operationTag = '---------------- Install npm dependencies';
        console.time(operationTag);
        const registry = `--registry=${config_1.npmRegistry}`;
        const npmOptions = [
            '--prefer-offline',
            '--no-audit',
            '--progress=false',
            registry,
        ];
        if (options === null || options === void 0 ? void 0 : options.isDependence) {
            npmOptions.push('--production');
        }
        let installProcess;
        if (yarnExists && (options === null || options === void 0 ? void 0 : options.runtime) !== types_1.RUNTIME.CI) {
            const addPackage = packageName ? ['add', packageName] : [];
            installProcess = cross_spawn_1.default('yarn', [...addPackage, registry], {
                cwd: targetDir,
                env: process.env,
                stdio: ['inherit', 'pipe', 'pipe'],
            });
        }
        else {
            installProcess = cross_spawn_1.default('npm', ['install', packageName, ...npmOptions], {
                cwd: targetDir,
                env: process.env,
                stdio: ['inherit', 'pipe', 'pipe'],
            });
        }
        installProcess.on('exit', () => console.timeEnd(operationTag));
        yield process_1.promisifyProcess(installProcess);
    });
}
exports.installDependencies = installDependencies;
function getMaterialNodeModulesPathList(dependencies = [], materialsDir) {
    const localPkg = util_1.getCurrentPackageJson();
    return dependencies.map(({ name, version }) => {
        const nameVersion = `${name}@${version}`;
        if (localPkg && localPkg.name === name && localPkg.version === version) {
            console.log('当前本地目录是素材库的时候，直接使用当前目录的 node_modules', nameVersion);
            return path_1.default.join(process.cwd(), 'node_modules');
        }
        return path_1.default.join(materialsDir, nameVersion, 'node_modules');
    });
}
exports.getMaterialNodeModulesPathList = getMaterialNodeModulesPathList;
function generateWebpackWebDevServerFile({ appBuildDir, buildTypeList, }) {
    return __awaiter(this, void 0, void 0, function* () {
        const dest = path_1.default.resolve(appBuildDir, `./webpack/devServer.js`);
        const template = yield fs_extra_1.default.readFile(path_1.default.resolve(config_2.appTemplateDir, './webpack/devServer.js'), {
            encoding: 'utf8',
        });
        const jsContent = lodash_template_1.default(template, {
            interpolate: /<%=([\s\S]+?)%>/g,
        })({
            isApp: buildTypeList.includes("app"),
            isAdminPortal: common_2.buildAsAdminPortalByBuildType(buildTypeList),
        });
        yield fs_extra_1.default.ensureFile(dest);
        yield fs_extra_1.default.writeFile(dest, jsContent);
    });
}
exports.generateWebpackWebDevServerFile = generateWebpackWebDevServerFile;
function downloadAssets(targetDir, assetUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        const isExist = fs_extra_1.default.existsSync(path_1.default.join(targetDir, 'package.json'));
        if (isExist) {
            return;
        }
        const [err, response] = yield util_1.promiseWrapper(axios_1.default({
            url: assetUrl,
            responseType: 'stream',
        }));
        if (err) {
            console.error('Fail to download weapps material package ' + assetUrl, err);
            throw err;
        }
        yield fs_extra_1.default.ensureDir(targetDir);
        const filename = common_1.getFileNameByUrl(assetUrl);
        const targetPath = path_1.default.resolve(targetDir, `./${filename}`);
        const writer = fs_extra_1.default.createWriteStream(targetPath);
        response.data.pipe(writer);
        return new Promise((resolve, reject) => {
            writer.on('finish', () => {
                console.log('下载成功', assetUrl);
                fs_extra_1.default.copy(targetDir, path_1.default.resolve(targetDir, '../preview'));
                resolve(targetPath);
            });
            writer.on('error', () => {
                console.error('下载失败', assetUrl);
                reject();
            });
        });
    });
}
exports.downloadAssets = downloadAssets;
