import { ActionType } from './action';
import { ICommonStyle } from './style';
import { IWeAppCode } from './lowcode';
import { IPlugin } from './plugins';
import { PropBindType, IEventModifiers, IAppAndPageVar, IWeAppComponentInstance } from './app';
export interface IWebRuntimeAppData extends IAppAndPageVar {
    envId?: string;
    datasources: any[];
    selectedPageId?: string;
    pageInstanceList: IPageInstance[];
    rootPath?: string;
    maxID?: number;
    codeModules: IWeAppCode[];
    npmDependencies: {
        [packageName: string]: string;
    };
    plugins?: IPlugin[];
    themeVars?: {
        [key: string]: string;
    };
    presetColors?: string[];
    appConfig?: {
        [key: string]: any;
    };
    extra: {
        domain: string;
    };
}
export interface IPageInstance extends ICommonInstanceProps, IAppAndPageVar {
    id: string;
    isHome?: boolean;
    componentSchemaJson?: IComponentSchemaJson;
    pluginInstances: IPluginInstance[];
    children?: IPageInstance[];
    codeModules: IWeAppCode[];
}
export interface IPageData {
    title: string;
    desc?: string;
}
export interface IDataBind {
    propertyPath: string;
    bindDataPath: string;
    type?: PropBindType;
}
export interface IDataType {
    propertyPath: string;
    type?: 'static' | 'slot' | 'bind';
}
export interface IDataAndBindInstanceProps {
    data: {
        [key: string]: any;
    };
    dataBinds?: IDataBind[];
    dataTypes?: IDataType[];
}
export interface IItemInstance extends IDataAndBindInstanceProps {
    key: Readonly<string>;
    sourceKey: string;
    instanceFunction?: string;
}
export interface IPluginInstance extends IItemInstance {
    instanceFunction?: string;
}
export interface IListenerInstance extends IItemInstance, IEventModifiers {
    trigger: string;
    target?: string | null;
    type: ActionType;
    handler?: {
        moduleName: string;
        name: string;
    };
    jsCode?: string;
}
export interface IComponentSchemaJson {
    key?: string;
    type?: string;
    ['x-component']?: string;
    ['x-props']?: IComponentInstanceProps;
    ['x-index']?: number;
    properties: {
        [key: string]: IComponentSchemaJson;
    };
    path?: string;
    genericComp?: IWeAppComponentInstance['genericComp'];
}
export interface ICommonInstanceProps extends IDataAndBindInstanceProps {
    style: object;
    commonStyle?: ICommonStyle;
    styleBindPath?: string;
    styleBind?: IDataBind;
    classNameListBind?: IDataBind;
    classNameList?: string[];
    listenerInstances: IListenerInstance[];
    customDataForm?: Record<string, any>;
    staticResourceAttribute?: string[];
}
export interface IComponentInstanceProps extends ICommonInstanceProps {
    sourceKey: string;
    isContainer: boolean;
}
export interface IPageWidgets {
    style: Record<string, any>;
    classList: string[];
    value?: any;
    [key: string]: any;
}
export interface II18nConfig {
    enabled: boolean;
    i18nData: any;
    baseUrl?: string;
    docUrl?: string;
}
export interface IExtraData {
    isComposite: boolean;
    compProps: any;
}
//# sourceMappingURL=web.d.ts.map